@@ -305,7 +305,6 @@ static int jlink_register_commands(struct command_context_s *cmd_ctx)
 
 static int jlink_init(void)
 {
-	int check_cnt;
 	int i;
 
 	jlink_jtag_handle = jlink_usb_open();
@@ -316,23 +315,27 @@ static int jlink_init(void)
 		return ERROR_JTAG_INIT_FAILED;
 	}
 
+	/*
+	 *  The next three instructions were added after discovering a problem while using an oscilloscope.  For the V8
+	 *	SAM-ICE dongle (and likely other j-link device variants), the reset line to the target microprocessor was found to
+	 *	cycle only intermittently during emulator startup (even after encountering the downstream reset instruction later
+	 *	in the code).  This was found to create two issues:  1) In general it is a bad practice to not reset a CPU to a known
+	 *	state when starting an emulator and 2) something critical happens inside the dongle when it does the first read
+	 *	following a new USB session.  Keeping the processor in reset during the first read collecting version information
+	 *	seems to prevent errant "J-Link command EMU_CMD_VERSION failed" issues.
+	 */
+
+	LOG_INFO("J-Link initialization started / target CPU reset initiated");
+	jlink_simple_command(EMU_CMD_HW_TRST0);
+	jlink_simple_command(EMU_CMD_HW_RESET0);
+	usleep(1000);
+
 	jlink_hw_jtag_version = 2;
-	check_cnt = 0;
-	while (check_cnt < 3)
-	{
-		if (jlink_get_version_info() == ERROR_OK)
-		{
-			/* attempt to get status */
-			jlink_get_status();
-			break;
-		}
 
-		check_cnt++;
-	}
-
-	if (check_cnt == 3)
+	if (jlink_get_version_info() == ERROR_OK)
 	{
-		LOG_INFO("J-Link initial read failed, don't worry");
+		/* attempt to get status */
+		jlink_get_status();
 	}
 
 	LOG_INFO("J-Link JTAG Interface ready");
@@ -496,12 +499,10 @@ static void jlink_reset(int trst, int srst)
 	{
 		jlink_simple_command(EMU_CMD_HW_TRST0);
 	}
+
 	if (trst == 0)
 	{
 		jlink_simple_command(EMU_CMD_HW_TRST1);
-		jtag_sleep(5000);
-		jlink_end_state(TAP_RESET);
-		jlink_state_move();
 	}
 }
 
@@ -832,20 +833,12 @@ static int jlink_tap_execute(void)
 	return ERROR_OK;
 }
 
-/*****************************************************************************/
-/* JLink USB low-level functions */
-
-static jlink_jtag_t* jlink_usb_open()
+static struct usb_device* find_jlink_device(void)
 {
 	struct usb_bus *busses;
 	struct usb_bus *bus;
 	struct usb_device *dev;
 
-	jlink_jtag_t *result;
-
-	result = (jlink_jtag_t*) malloc(sizeof(jlink_jtag_t));
-
-	usb_init();
 	usb_find_busses();
 	usb_find_devices();
 
@@ -857,36 +850,102 @@ static jlink_jtag_t* jlink_usb_open()
 	{
 		for (dev = bus->devices; dev; dev = dev->next)
 		{
-			if ((dev->descriptor.idVendor == VID) && (dev->descriptor.idProduct == PID))
-			{
-				result->usb_handle = usb_open(dev);
+			if ((dev->descriptor.idVendor == VID) && (dev->descriptor.idProduct == PID)) {
+				return dev;
+			}
+		}
+	}
 
-				/* usb_set_configuration required under win32 */
-				usb_set_configuration(result->usb_handle, dev->config[0].bConfigurationValue);
-				usb_claim_interface(result->usb_handle, 0);
+	return NULL;
+}
+
+/*****************************************************************************/
+/* JLink USB low-level functions */
+
+static jlink_jtag_t* jlink_usb_open()
+{
+	struct usb_device *dev;
+
+	jlink_jtag_t *result;
+
+	result = (jlink_jtag_t*) malloc(sizeof(jlink_jtag_t));
+
+	usb_init();
+
+	if ((dev = find_jlink_device()) == NULL) {
+		free(result);
+		return NULL;
+	}
+
+	result->usb_handle = usb_open(dev);
+
+	if (result->usb_handle)
+	{
+
+		/* BE ***VERY CAREFUL*** ABOUT MAKING CHANGES IN THIS AREA!!!!!!!!!!!
+		 * The behavior of libusb is not completely consistent across Windows, Linux, and Mac OS X platforms.  The actions taken
+		 * in the following compiler conditionals may not agree with published documentation for libusb, but were found
+		 * to be necessary through trials and tribulations.  Even little tweaks can break one or more platforms, so if you do make changes
+		 * test them carefully on all platforms before committing them!
+		 */
+
+#if IS_WIN32 == 0
+
+		usb_reset(result->usb_handle);
+
+#if IS_DARWIN == 0
+
+		int timeout = 5;
+
+		/* reopen jlink after usb_reset
+		 * on win32 this may take a second or two to re-enumerate */
+		while ((dev = find_jlink_device()) == NULL)
+		{
+			usleep(1000);
+			timeout--;
+			if (!timeout) {
+				break;
+			}
+		}
+
+		if (dev == NULL)
+		{
+			free(result);
+			return NULL;
+		}
+
+		result->usb_handle = usb_open(dev);
+#endif
+
+#endif
+
+		if (result->usb_handle)
+		{
+			/* usb_set_configuration required under win32 */
+			usb_set_configuration(result->usb_handle, dev->config[0].bConfigurationValue);
+			usb_claim_interface(result->usb_handle, 0);
 
 #if 0
-				/*
-				 * This makes problems under Mac OS X. And is not needed
-				 * under Windows. Hopefully this will not break a linux build
-				 */
-				usb_set_altinterface(result->usb_handle, 0);
+			/*
+			 * This makes problems under Mac OS X. And is not needed
+			 * under Windows. Hopefully this will not break a linux build
+			 */
+			usb_set_altinterface(result->usb_handle, 0);
 #endif
-				struct usb_interface *iface = dev->config->interface;
-				struct usb_interface_descriptor *desc = iface->altsetting;
-				for (int i = 0; i < desc->bNumEndpoints; i++)
-				{
-					uint8_t epnum = desc->endpoint[i].bEndpointAddress;
-					bool is_input = epnum & 0x80;
-					LOG_DEBUG("usb ep %s %02x", is_input ? "in" : "out", epnum);
-					if (is_input)
-						jlink_read_ep = epnum;
-					else
-						jlink_write_ep = epnum;
-				}
-
-				return result;
+			struct usb_interface *iface = dev->config->interface;
+			struct usb_interface_descriptor *desc = iface->altsetting;
+			for (int i = 0; i < desc->bNumEndpoints; i++)
+			{
+				uint8_t epnum = desc->endpoint[i].bEndpointAddress;
+				bool is_input = epnum & 0x80;
+				LOG_DEBUG("usb ep %s %02x", is_input ? "in" : "out", epnum);
+				if (is_input)
+					jlink_read_ep = epnum;
+				else
+					jlink_write_ep = epnum;
 			}
+
+			return result;
 		}
 	}
 
