@@ -2,7 +2,7 @@
  *   Copyright (C) 2005 by Dominic Rath                                    *
  *   Dominic.Rath@gmx.de                                                   *
  *                                                                         *
- *   Copyright (C) 2007,2008 yvind Harboe                                 *
+ *   Copyright (C) 2007,2008 Øyvind Harboe                                 *
  *   oyvind.harboe@zylin.com                                               *
  *                                                                         *
  *   Copyright (C) 2009 SoftPLC Corporation                                *
@@ -34,142 +34,21 @@
 #include "jtag.h"
 #include "minidriver.h"
 #include "interface.h"
+#include "interfaces.h"
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
 
 static const Jim_Nvp nvp_jtag_tap_event[] = {
+	{ .value = JTAG_TRST_ASSERTED,		.name = "post-reset" },
+	{ .value = JTAG_TAP_EVENT_SETUP,	.name = "setup" },
 	{ .value = JTAG_TAP_EVENT_ENABLE,       .name = "tap-enable" },
 	{ .value = JTAG_TAP_EVENT_DISABLE,      .name = "tap-disable" },
 
 	{ .name = NULL, .value = -1 }
 };
 
-/* jtag interfaces (parport, FTDI-USB, TI-USB, ...)
- */
-
-#if BUILD_ZY1000 == 1
-	extern jtag_interface_t zy1000_interface;
-#elif defined(BUILD_MINIDRIVER_DUMMY)
-	extern jtag_interface_t minidummy_interface;
-#else // standard drivers
-#if BUILD_PARPORT == 1
-	extern jtag_interface_t parport_interface;
-#endif
-
-#if BUILD_DUMMY == 1
-	extern jtag_interface_t dummy_interface;
-#endif
-
-#if BUILD_FT2232_FTD2XX == 1
-	extern jtag_interface_t ft2232_interface;
-#endif
-
-#if BUILD_FT2232_LIBFTDI == 1
-	extern jtag_interface_t ft2232_interface;
-#endif
-
-#if BUILD_AMTJTAGACCEL == 1
-	extern jtag_interface_t amt_jtagaccel_interface;
-#endif
-
-#if BUILD_EP93XX == 1
-	extern jtag_interface_t ep93xx_interface;
-#endif
-
-#if BUILD_AT91RM9200 == 1
-	extern jtag_interface_t at91rm9200_interface;
-#endif
-
-#if BUILD_GW16012 == 1
-	extern jtag_interface_t gw16012_interface;
-#endif
-
-#if BUILD_PRESTO_LIBFTDI == 1 || BUILD_PRESTO_FTD2XX == 1
-	extern jtag_interface_t presto_interface;
-#endif
-
-#if BUILD_USBPROG == 1
-	extern jtag_interface_t usbprog_interface;
-#endif
-
-#if BUILD_JLINK == 1
-	extern jtag_interface_t jlink_interface;
-#endif
-
-#if BUILD_VSLLINK == 1
-	extern jtag_interface_t vsllink_interface;
-#endif
-
-#if BUILD_RLINK == 1
-	extern jtag_interface_t rlink_interface;
-#endif
-
-#if BUILD_ARMJTAGEW == 1
-	extern jtag_interface_t armjtagew_interface;
-#endif
-#endif // standard drivers
-
-/**
- * The list of built-in JTAG interfaces, containing entries for those
- * drivers that were enabled by the @c configure script.
- *
- * The list should be defined to contain either one minidriver interface
- * or some number of standard driver interfaces, never both.
- */
-jtag_interface_t *jtag_interfaces[] = {
-#if BUILD_ZY1000 == 1
-	&zy1000_interface,
-#elif defined(BUILD_MINIDRIVER_DUMMY)
-	&minidummy_interface,
-#else // standard drivers
-#if BUILD_PARPORT == 1
-	&parport_interface,
-#endif
-#if BUILD_DUMMY == 1
-	&dummy_interface,
-#endif
-#if BUILD_FT2232_FTD2XX == 1
-	&ft2232_interface,
-#endif
-#if BUILD_FT2232_LIBFTDI == 1
-	&ft2232_interface,
-#endif
-#if BUILD_AMTJTAGACCEL == 1
-	&amt_jtagaccel_interface,
-#endif
-#if BUILD_EP93XX == 1
-	&ep93xx_interface,
-#endif
-#if BUILD_AT91RM9200 == 1
-	&at91rm9200_interface,
-#endif
-#if BUILD_GW16012 == 1
-	&gw16012_interface,
-#endif
-#if BUILD_PRESTO_LIBFTDI == 1 || BUILD_PRESTO_FTD2XX == 1
-	&presto_interface,
-#endif
-#if BUILD_USBPROG == 1
-	&usbprog_interface,
-#endif
-#if BUILD_JLINK == 1
-	&jlink_interface,
-#endif
-#if BUILD_VSLLINK == 1
-	&vsllink_interface,
-#endif
-#if BUILD_RLINK == 1
-	&rlink_interface,
-#endif
-#if BUILD_ARMJTAGEW == 1
-	&armjtagew_interface,
-#endif
-#endif // standard drivers
-	NULL,
-};
-
 extern jtag_interface_t *jtag_interface;
 
 /* jtag commands */
@@ -178,10 +57,13 @@ static int handle_interface_list_command(struct command_context_s *cmd_ctx,
 static int handle_interface_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_jtag_speed_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_jtag_khz_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
+static int handle_jtag_rclk_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_jtag_device_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_reset_config_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_jtag_nsrst_delay_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_jtag_ntrst_delay_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
+static int handle_jtag_nsrst_assert_width_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
+static int handle_jtag_ntrst_assert_width_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 
 static int handle_scan_chain_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 
@@ -196,8 +78,6 @@ static int handle_verify_ircapture_command(struct command_context_s *cmd_ctx, ch
 static int handle_verify_jtag_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 static int handle_tms_sequence_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc);
 
-extern int jtag_examine_chain(void);
-extern int jtag_validate_chain(void);
 
 enum jtag_tap_cfg_param {
 	JCFG_EVENT
@@ -263,9 +143,11 @@ static int jtag_tap_configure_cmd(Jim_GetOptInfo *goi, jtag_tap_t * tap)
 					}
 
 					if (goi->isconfigure) {
+						bool replace = true;
 						if (jteap == NULL) {
 							/* create new */
 							jteap = calloc(1, sizeof (*jteap));
+							replace = false;
 						}
 						jteap->event = n->value;
 						Jim_GetOpt_Obj(goi, &o);
@@ -275,9 +157,12 @@ static int jtag_tap_configure_cmd(Jim_GetOptInfo *goi, jtag_tap_t * tap)
 						jteap->body = Jim_DuplicateObj(goi->interp, o);
 						Jim_IncrRefCount(jteap->body);
 
-						/* add to head of event list */
-						jteap->next = tap->event_action;
-						tap->event_action = jteap;
+						if (!replace)
+						{
+							/* add to head of event list */
+							jteap->next = tap->event_action;
+							tap->event_action = jteap;
+						}
 						Jim_SetEmptyResult(goi->interp);
 					} else {
 						/* get */
@@ -312,7 +197,6 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 	jim_wide w;
 	int x;
 	int e;
-	int reqbits;
 	Jim_Nvp *n;
 	char *cp;
 	const Jim_Nvp opts[] = {
@@ -342,6 +226,7 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 	 * */
 	if (goi->argc < 3) {
 		Jim_SetResult_sprintf(goi->interp, "Missing CHIP TAP OPTIONS ....");
+		free(pTap);
 		return JIM_ERR;
 	}
 	Jim_GetOpt_String(goi, &cp, NULL);
@@ -359,18 +244,19 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 	LOG_DEBUG("Creating New Tap, Chip: %s, Tap: %s, Dotted: %s, %d params",
 			  pTap->chip, pTap->tapname, pTap->dotted_name, goi->argc);
 
-	/* deal with options */
-#define NTREQ_IRLEN      1
-#define NTREQ_IRCAPTURE  2
-#define NTREQ_IRMASK     4
-
-	/* clear them as we find them */
-	reqbits = (NTREQ_IRLEN | NTREQ_IRCAPTURE | NTREQ_IRMASK);
+	/* IEEE specifies that the two LSBs of an IR scan are 01, so make
+	 * that the default.  The "-irlen" and "-irmask" options are only
+	 * needed to cope with nonstandard TAPs, or to specify more bits.
+	 */
+	pTap->ir_capture_mask = 0x03;
+	pTap->ir_capture_value = 0x01;
 
 	while (goi->argc) {
 		e = Jim_GetOpt_Nvp(goi, opts, &n);
 		if (e != JIM_OK) {
 			Jim_GetOpt_NvpUnknown(goi, opts, 0);
+			free((void *)pTap->dotted_name);
+			free(pTap);
 			return e;
 		}
 		LOG_DEBUG("Processing option: %s", n->name);
@@ -388,12 +274,16 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 			e = Jim_GetOpt_Wide(goi, &w);
 			if (e != JIM_OK) {
 				Jim_SetResult_sprintf(goi->interp, "option: %s bad parameter", n->name);
+				free((void *)pTap->dotted_name);
+				free(pTap);
 				return e;
 			}
 
 			new_expected_ids = malloc(sizeof(uint32_t) * (pTap->expected_ids_cnt + 1));
 			if (new_expected_ids == NULL) {
 				Jim_SetResult_sprintf(goi->interp, "no memory");
+				free((void *)pTap->dotted_name);
+				free(pTap);
 				return JIM_ERR;
 			}
 
@@ -412,32 +302,45 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 			e = Jim_GetOpt_Wide(goi, &w);
 			if (e != JIM_OK) {
 				Jim_SetResult_sprintf(goi->interp, "option: %s bad parameter", n->name);
+				free((void *)pTap->dotted_name);
+				free(pTap);
 				return e;
 			}
 			switch (n->value) {
 			case NTAP_OPT_IRLEN:
 				if (w > (jim_wide) (8 * sizeof(pTap->ir_capture_value)))
-					LOG_WARNING("huge IR length %d", (int) w);
+					LOG_WARNING("%s: huge IR length %d",
+							pTap->dotted_name,
+							(int) w);
 				pTap->ir_length = w;
-				reqbits &= (~(NTREQ_IRLEN));
 				break;
 			case NTAP_OPT_IRMASK:
 				if (is_bad_irval(pTap->ir_length, w)) {
-					LOG_ERROR("IR mask %x too big",
+					LOG_ERROR("%s: IR mask %x too big",
+							pTap->dotted_name,
 							(int) w);
+					free((void *)pTap->dotted_name);
+					free(pTap);
 					return ERROR_FAIL;
 				}
+				if ((w & 3) != 3)
+					LOG_WARNING("%s: nonstandard IR mask",
+							pTap->dotted_name);
 				pTap->ir_capture_mask = w;
-				reqbits &= (~(NTREQ_IRMASK));
 				break;
 			case NTAP_OPT_IRCAPTURE:
 				if (is_bad_irval(pTap->ir_length, w)) {
-					LOG_ERROR("IR capture %x too big",
+					LOG_ERROR("%s: IR capture %x too big",
+							pTap->dotted_name,
 							(int) w);
+					free((void *)pTap->dotted_name);
+					free(pTap);
 					return ERROR_FAIL;
 				}
+				if ((w & 3) != 1)
+					LOG_WARNING("%s: nonstandard IR value",
+							pTap->dotted_name);
 				pTap->ir_capture_value = w;
-				reqbits &= (~(NTREQ_IRCAPTURE));
 				break;
 			}
 		} /* switch (n->value) */
@@ -447,14 +350,14 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 	pTap->enabled = !pTap->disabled_after_reset;
 
 	/* Did all the required option bits get cleared? */
-	if (0 == reqbits)
+	if (pTap->ir_length != 0)
 	{
 		jtag_tap_init(pTap);
 		return ERROR_OK;
 	}
 
 	Jim_SetResult_sprintf(goi->interp,
-			"newtap: %s missing required parameters",
+			"newtap: %s missing IR length",
 			pTap->dotted_name);
 	jtag_tap_free(pTap);
 	return JIM_ERR;
@@ -463,37 +366,31 @@ static int jim_newtap_cmd(Jim_GetOptInfo *goi)
 static void jtag_tap_handle_event(jtag_tap_t *tap, enum jtag_event e)
 {
 	jtag_tap_event_action_t * jteap;
-	int done;
 
-	jteap = tap->event_action;
-
-	done = 0;
-	while (jteap) {
+	for (jteap = tap->event_action; jteap != NULL; jteap = jteap->next) {
 		if (jteap->event == e) {
-			done = 1;
-			LOG_DEBUG("JTAG tap: %s event: %d (%s) action: %s\n",
+			LOG_DEBUG("JTAG tap: %s event: %d (%s)\n\taction: %s",
 					tap->dotted_name,
 					e,
 					Jim_Nvp_value2name_simple(nvp_jtag_tap_event, e)->name,
 					Jim_GetString(jteap->body, NULL));
 			if (Jim_EvalObj(interp, jteap->body) != JIM_OK) {
 				Jim_PrintErrorMessage(interp);
-			} else {
+			} else switch (e) {
+			case JTAG_TAP_EVENT_ENABLE:
+			case JTAG_TAP_EVENT_DISABLE:
 				/* NOTE:  we currently assume the handlers
-				 * can't fail.  That presumes later code
-				 * will be verifying the scan chains ...
+				 * can't fail.  Right here is where we should
+				 * really be verifying the scan chains ...
 				 */
 				tap->enabled = (e == JTAG_TAP_EVENT_ENABLE);
+				LOG_INFO("JTAG tap: %s %s", tap->dotted_name,
+					tap->enabled ? "enabled" : "disabled");
+				break;
+			default:
+				break;
 			}
 		}
-
-		jteap = jteap->next;
-	}
-
-	if (!done) {
-		LOG_DEBUG("event %d %s - no action",
-				e,
-				Jim_Nvp_value2name_simple(nvp_jtag_tap_event, e)->name);
 	}
 }
 
@@ -508,17 +405,20 @@ static int jim_jtag_command(Jim_Interp *interp, int argc, Jim_Obj *const *argv)
 
 	enum {
 		JTAG_CMD_INTERFACE,
+		JTAG_CMD_INIT,
 		JTAG_CMD_INIT_RESET,
 		JTAG_CMD_NEWTAP,
 		JTAG_CMD_TAPENABLE,
 		JTAG_CMD_TAPDISABLE,
 		JTAG_CMD_TAPISENABLED,
 		JTAG_CMD_CONFIGURE,
-		JTAG_CMD_CGET
+		JTAG_CMD_CGET,
+		JTAG_CMD_NAMES,
 	};
 
 	const Jim_Nvp jtag_cmds[] = {
 		{ .name = "interface"     , .value = JTAG_CMD_INTERFACE },
+		{ .name = "arp_init"      , .value = JTAG_CMD_INIT },
 		{ .name = "arp_init-reset", .value = JTAG_CMD_INIT_RESET },
 		{ .name = "newtap"        , .value = JTAG_CMD_NEWTAP },
 		{ .name = "tapisenabled"     , .value = JTAG_CMD_TAPISENABLED },
@@ -526,6 +426,7 @@ static int jim_jtag_command(Jim_Interp *interp, int argc, Jim_Obj *const *argv)
 		{ .name = "tapdisable"    , .value = JTAG_CMD_TAPDISABLE },
 		{ .name = "configure"     , .value = JTAG_CMD_CONFIGURE },
 		{ .name = "cget"          , .value = JTAG_CMD_CGET },
+		{ .name = "names"         , .value = JTAG_CMD_NAMES },
 
 		{ .name = NULL, .value = -1 },
 	};
@@ -551,6 +452,17 @@ static int jim_jtag_command(Jim_Interp *interp, int argc, Jim_Obj *const *argv)
 		}
 		Jim_SetResultString(goi.interp, jtag_interface->name, -1);
 		return JIM_OK;
+	case JTAG_CMD_INIT:
+		if (goi.argc != 0) {
+			Jim_WrongNumArgs(goi.interp, 1, goi.argv-1, "(no params)");
+			return JIM_ERR;
+		}
+		e = jtag_init_inner(context);
+		if (e != ERROR_OK) {
+			Jim_SetResult_sprintf(goi.interp, "error: %d", e);
+			return JIM_ERR;
+		}
+		return JIM_OK;
 	case JTAG_CMD_INIT_RESET:
 		if (goi.argc != 0) {
 			Jim_WrongNumArgs(goi.interp, 1, goi.argv-1, "(no params)");
@@ -620,7 +532,8 @@ static int jim_jtag_command(Jim_Interp *interp, int argc, Jim_Obj *const *argv)
 
 	case JTAG_CMD_CGET:
 		if (goi.argc < 2) {
-			Jim_WrongNumArgs(goi.interp, 0, NULL, "?tap-name? -option ...");
+			Jim_WrongNumArgs(goi.interp, 0, NULL,
+					"cget tap_name queryparm");
 			return JIM_ERR;
 		}
 
@@ -640,7 +553,8 @@ static int jim_jtag_command(Jim_Interp *interp, int argc, Jim_Obj *const *argv)
 
 	case JTAG_CMD_CONFIGURE:
 		if (goi.argc < 3) {
-			Jim_WrongNumArgs(goi.interp, 0, NULL, "?tap-name? -option ?VALUE? ...");
+			Jim_WrongNumArgs(goi.interp, 0, NULL,
+					"configure tap_name attribute value ...");
 			return JIM_ERR;
 		}
 
@@ -656,11 +570,42 @@ static int jim_jtag_command(Jim_Interp *interp, int argc, Jim_Obj *const *argv)
 			goi.isconfigure = 1;
 			return jtag_tap_configure_cmd(&goi, t);
 		}
+		break;
+
+	case JTAG_CMD_NAMES:
+		if (goi.argc != 0) {
+			Jim_WrongNumArgs(goi.interp, 1, goi.argv, "Too many parameters");
+			return JIM_ERR;
+		}
+		Jim_SetResult(goi.interp, Jim_NewListObj(goi.interp, NULL, 0));
+		{
+			jtag_tap_t *tap;
+
+			for (tap = jtag_all_taps(); tap; tap = tap->next_tap) {
+				Jim_ListAppendElement(goi.interp,
+					Jim_GetResult(goi.interp),
+					Jim_NewStringObj(goi.interp,
+						tap->dotted_name, -1));
+			}
+			return JIM_OK;
+		}
+		break;
+
 	}
 
 	return JIM_ERR;
 }
 
+
+void jtag_notify_event(enum jtag_event event)
+{
+	jtag_tap_t *tap;
+
+	for (tap = jtag_all_taps(); tap; tap = tap->next_tap)
+		jtag_tap_handle_event(tap, event);
+}
+
+
 int jtag_register_commands(struct command_context_s *cmd_ctx)
 {
 	register_jim(cmd_ctx, "jtag", jim_jtag_command, "perform jtag tap actions");
@@ -675,15 +620,26 @@ int jtag_register_commands(struct command_context_s *cmd_ctx)
 	register_command(cmd_ctx, NULL, "jtag_khz", handle_jtag_khz_command,
 		COMMAND_ANY, "set maximum jtag speed (if supported); "
 		"parameter is maximum khz, or 0 for adaptive clocking (RTCK).");
+	register_command(cmd_ctx, NULL, "jtag_rclk", handle_jtag_rclk_command,
+		COMMAND_ANY, "fallback_speed_khz - set JTAG speed to RCLK or use fallback speed");
 	register_command(cmd_ctx, NULL, "jtag_device", handle_jtag_device_command,
 		COMMAND_CONFIG, "(DEPRECATED) jtag_device <ir_length> <ir_expected> <ir_mask>");
 	register_command(cmd_ctx, NULL, "reset_config", handle_reset_config_command,
 		COMMAND_ANY,
-		"[none/trst_only/srst_only/trst_and_srst] [srst_pulls_trst/trst_pulls_srst] [combined/separate] [trst_push_pull/trst_open_drain] [srst_push_pull/srst_open_drain]");
+		"reset_config "
+		"[none|trst_only|srst_only|trst_and_srst] "
+		"[srst_pulls_trst|trst_pulls_srst|combined|separate] "
+		"[srst_gates_jtag|srst_nogate] "
+		"[trst_push_pull|trst_open_drain] "
+		"[srst_push_pull|srst_open_drain]");
 	register_command(cmd_ctx, NULL, "jtag_nsrst_delay", handle_jtag_nsrst_delay_command,
 		COMMAND_ANY, "jtag_nsrst_delay <ms> - delay after deasserting srst in ms");
 	register_command(cmd_ctx, NULL, "jtag_ntrst_delay", handle_jtag_ntrst_delay_command,
 		COMMAND_ANY, "jtag_ntrst_delay <ms> - delay after deasserting trst in ms");
+	register_command(cmd_ctx, NULL, "jtag_nsrst_assert_width", handle_jtag_nsrst_assert_width_command,
+		COMMAND_ANY, "jtag_nsrst_assert_width <ms> - delay after asserting srst in ms");
+	register_command(cmd_ctx, NULL, "jtag_ntrst_assert_width", handle_jtag_ntrst_assert_width_command,
+		COMMAND_ANY, "jtag_ntrst_assert_width <ms> - delay after asserting trst in ms");
 
 	register_command(cmd_ctx, NULL, "scan_chain", handle_scan_chain_command,
 		COMMAND_EXEC, "print current scan chain configuration");
@@ -895,9 +851,6 @@ static int handle_reset_config_command(struct command_context_s *cmd_ctx, char *
 	int new_cfg = 0;
 	int mask = 0;
 
-	if (argc < 1)
-		return ERROR_COMMAND_SYNTAX_ERROR;
-
 	/* Original versions cared about the order of these tokens:
 	 *   reset_config signals [combination [trst_type [srst_type]]]
 	 * They also clobbered the previous configuration even on error.
@@ -909,6 +862,22 @@ static int handle_reset_config_command(struct command_context_s *cmd_ctx, char *
 		int tmp = 0;
 		int m;
 
+		/* gating */
+		m = RESET_SRST_NO_GATING;
+		if (strcmp(*args, "srst_gates_jtag") == 0)
+			/* default: don't use JTAG while SRST asserted */;
+		else if (strcmp(*args, "srst_nogate") == 0)
+			tmp = RESET_SRST_NO_GATING;
+		else
+			m = 0;
+		if (mask & m) {
+			LOG_ERROR("extra reset_config %s spec (%s)",
+					"gating", *args);
+			return ERROR_INVALID_ARGUMENTS;
+		}
+		if (m)
+			goto next;
+
 		/* signals */
 		m = RESET_HAS_TRST | RESET_HAS_SRST;
 		if (strcmp(*args, "none") == 0)
@@ -994,10 +963,81 @@ next:
 	}
 
 	/* clear previous values of those bits, save new values */
-	enum reset_types old_cfg = jtag_get_reset_config();
-	old_cfg &= ~mask;
-	new_cfg |= old_cfg;
-	jtag_set_reset_config(new_cfg);
+	if (mask) {
+		int old_cfg = jtag_get_reset_config();
+
+		old_cfg &= ~mask;
+		new_cfg |= old_cfg;
+		jtag_set_reset_config(new_cfg);
+	} else
+		new_cfg = jtag_get_reset_config();
+
+
+	/*
+	 * Display the (now-)current reset mode
+	 */
+	char *modes[5];
+
+	/* minimal JTAG has neither SRST nor TRST (so that's the default) */
+	switch (new_cfg & (RESET_HAS_TRST | RESET_HAS_SRST)) {
+	case RESET_HAS_SRST:
+		modes[0] = "srst_only";
+		break;
+	case RESET_HAS_TRST:
+		modes[0] = "trst_only";
+		break;
+	case RESET_TRST_AND_SRST:
+		modes[0] = "trst_and_srst";
+		break;
+	default:
+		modes[0] = "none";
+		break;
+	}
+
+	/* normally SRST and TRST are decoupled; but bugs happen ... */
+	switch (new_cfg & (RESET_SRST_PULLS_TRST | RESET_TRST_PULLS_SRST)) {
+	case RESET_SRST_PULLS_TRST:
+		modes[1] = "srst_pulls_trst";
+		break;
+	case RESET_TRST_PULLS_SRST:
+		modes[1] = "trst_pulls_srst";
+		break;
+	case RESET_SRST_PULLS_TRST | RESET_TRST_PULLS_SRST:
+		modes[1] = "combined";
+		break;
+	default:
+		modes[1] = "separate";
+		break;
+	}
+
+	/* TRST-less connectors include Altera, Xilinx, and minimal JTAG */
+	if (new_cfg & RESET_HAS_TRST) {
+		if (new_cfg & RESET_TRST_OPEN_DRAIN)
+			modes[3] = " trst_open_drain";
+		else
+			modes[3] = " trst_push_pull";
+	} else
+		modes[3] = "";
+
+	/* SRST-less connectors include TI-14, Xilinx, and minimal JTAG */
+	if (new_cfg & RESET_HAS_SRST) {
+		if (new_cfg & RESET_SRST_NO_GATING)
+			modes[2] = " srst_nogate";
+		else
+			modes[2] = " srst_gates_jtag";
+
+		if (new_cfg & RESET_SRST_PUSH_PULL)
+			modes[4] = " srst_push_pull";
+		else
+			modes[4] = " srst_open_drain";
+	} else {
+		modes[2] = "";
+		modes[4] = "";
+	}
+
+	command_print(cmd_ctx, "%s %s%s%s%s",
+			modes[0], modes[1],
+			modes[2], modes[3], modes[4]);
 
 	return ERROR_OK;
 }
@@ -1036,10 +1076,47 @@ static int handle_jtag_ntrst_delay_command(struct command_context_s *cmd_ctx,
 	return ERROR_OK;
 }
 
+static int handle_jtag_nsrst_assert_width_command(struct command_context_s *cmd_ctx,
+		char *cmd, char **args, int argc)
+{
+	if (argc > 1)
+		return ERROR_COMMAND_SYNTAX_ERROR;
+	if (argc == 1)
+	{
+		unsigned delay;
+		int retval = parse_uint(args[0], &delay);
+		if (ERROR_OK != retval)
+			return retval;
+		jtag_set_nsrst_assert_width(delay);
+	}
+	command_print(cmd_ctx, "jtag_nsrst_assert_width: %u", jtag_get_nsrst_assert_width());
+	return ERROR_OK;
+}
+
+static int handle_jtag_ntrst_assert_width_command(struct command_context_s *cmd_ctx,
+		char *cmd, char **args, int argc)
+{
+	if (argc > 1)
+		return ERROR_COMMAND_SYNTAX_ERROR;
+	if (argc == 1)
+	{
+		unsigned delay;
+		int retval = parse_uint(args[0], &delay);
+		if (ERROR_OK != retval)
+			return retval;
+		jtag_set_ntrst_assert_width(delay);
+	}
+	command_print(cmd_ctx, "jtag_ntrst_assert_width: %u", jtag_get_ntrst_assert_width());
+	return ERROR_OK;
+}
+
 static int handle_jtag_speed_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
 {
 	int retval = ERROR_OK;
 
+	command_print(cmd_ctx, "OLD SYNTAX: DEPRECATED - "
+			"use jtag_khz, not jtag_speed");
+
 	if (argc > 1)
 		return ERROR_COMMAND_SYNTAX_ERROR;
 	if (argc == 1)
@@ -1050,7 +1127,7 @@ static int handle_jtag_speed_command(struct command_context_s *cmd_ctx, char *cm
 		int retval = parse_uint(args[0], &cur_speed);
 		if (ERROR_OK != retval)
 			return retval;
-		retval = jtag_set_speed(cur_speed);
+		retval = jtag_config_speed(cur_speed);
 
 	}
 	command_print(cmd_ctx, "jtag_speed: %d", jtag_get_speed());
@@ -1088,6 +1165,36 @@ static int handle_jtag_khz_command(struct command_context_s *cmd_ctx, char *cmd,
 	return retval;
 }
 
+static int handle_jtag_rclk_command(struct command_context_s *cmd_ctx, char *cmd, char **args, int argc)
+{
+	if (argc > 1)
+		return ERROR_COMMAND_SYNTAX_ERROR;
+
+	int retval = ERROR_OK;
+	if (argc == 1)
+	{
+		unsigned khz = 0;
+		int retval = parse_uint(args[0], &khz);
+		if (ERROR_OK != retval)
+			return retval;
+		retval = jtag_config_rclk(khz);
+		if (ERROR_OK != retval)
+			return retval;
+	}
+
+	int cur_khz = jtag_get_speed_khz();
+	retval = jtag_get_speed_readable(&cur_khz);
+	if (ERROR_OK != retval)
+		return retval;
+
+	if (cur_khz)
+		command_print(cmd_ctx, "RCLK not supported - fallback to %d kHz", cur_khz);
+	else
+		command_print(cmd_ctx, "RCLK - adaptive");
+
+	return retval;
+}
+
 static int handle_jtag_reset_command(struct command_context_s *cmd_ctx,
 		char *cmd, char **args, int argc)
 {
@@ -1114,9 +1221,7 @@ static int handle_jtag_reset_command(struct command_context_s *cmd_ctx,
 		return ERROR_JTAG_INIT_FAILED;
 
 	jtag_add_reset(trst, srst);
-	jtag_execute_queue();
-
-	return ERROR_OK;
+	return jtag_execute_queue();
 }
 
 static int handle_runtest_command(struct command_context_s *cmd_ctx,
@@ -1131,15 +1236,15 @@ static int handle_runtest_command(struct command_context_s *cmd_ctx,
 		return retval;
 
 	jtag_add_runtest(num_clocks, TAP_IDLE);
-	jtag_execute_queue();
-
-	return ERROR_OK;
+	return jtag_execute_queue();
 }
 
 /*
  * For "irscan" or "drscan" commands, the "end" (really, "next") state
  * should be stable ... and *NOT* a shift state, otherwise free-running
  * jtag clocks could change the values latched by the update state.
+ * Not surprisingly, this is the same constraint as SVF; the "irscan"
+ * and "drscan" commands are a write-only subset of what SVF provides.
  */
 static bool scan_is_safe(tap_state_t state)
 {
@@ -1177,25 +1282,14 @@ static int handle_irscan_command(struct command_context_s *cmd_ctx, char *cmd, c
 	if (argc >= 4) {
 		/* have at least one pair of numbers. */
 		/* is last pair the magic text? */
-		if (0 == strcmp("-endstate", args[ argc - 2 ])) {
-			const char *cpA;
-			const char *cpS;
-			cpA = args[ argc-1 ];
-			for (endstate = 0 ; endstate < TAP_NUM_STATES ; endstate++) {
-				cpS = tap_state_name(endstate);
-				if (0 == strcmp(cpA, cpS)) {
-					break;
-				}
-			}
-			if (endstate >= TAP_NUM_STATES) {
+		if (strcmp("-endstate", args[argc - 2]) == 0) {
+			endstate = tap_state_by_name(args[argc - 1]);
+			if (endstate == TAP_INVALID)
 				return ERROR_COMMAND_SYNTAX_ERROR;
-			} else {
-				if (!scan_is_safe(endstate))
-					LOG_WARNING("irscan with unsafe "
-							"endstate \"%s\"", cpA);
-				/* found - remove the last 2 args */
-				argc -= 2;
-			}
+			if (!scan_is_safe(endstate))
+				LOG_WARNING("unstable irscan endstate \"%s\"",
+						args[argc - 1]);
+			argc -= 2;
 		}
 	}
 
@@ -1210,7 +1304,12 @@ static int handle_irscan_command(struct command_context_s *cmd_ctx, char *cmd, c
 		tap = jtag_tap_by_string(args[i*2]);
 		if (tap == NULL)
 		{
+			int j;
+			for (j = 0; j < i; j++)
+				free(fields[j].out_value);
+                        free(fields);
 			command_print(cmd_ctx, "Tap: %s unknown", args[i*2]);
+
 			return ERROR_FAIL;
 		}
 		int field_size = tap->ir_length;
