@@ -1,22 +1,21 @@
-/***************************************************************************
- *	 Copyright (C) 2009 by Simon Qian									   *
- *	 SimonQian@SimonQian.com											   *
- *                                                                         *
- *	 This program is free software; you can redistribute it and/or modify  *
- *	 it under the terms of the GNU General Public License as published by  *
- *	 the Free Software Foundation; either version 2 of the License, or	   *
- *	 (at your option) any later version.								   *
- *																		   *
- *	 This program is distributed in the hope that it will be useful,	   *
- *	 but WITHOUT ANY WARRANTY; without even the implied warranty of		   *
- *	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		   *
- *	 GNU General Public License for more details.						   *
- *																		   *
- *	 You should have received a copy of the GNU General Public License	   *
- *	 along with this program; if not, write to the						   *
- *	 Free Software Foundation, Inc.,									   *
- *	 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.			   *
- ***************************************************************************/
+/*
+ * Copyright (C) 2009 by Simon Qian
+ * SimonQian@SimonQian.com
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 
 /* The specification for SVF is available here:
@@ -32,8 +31,8 @@
 #include "config.h"
 #endif
 
-#include "svf.h"
 #include "jtag.h"
+#include "svf.h"
 #include "time_support.h"
 
 
@@ -56,7 +55,7 @@ typedef enum
 	TRST,
 }svf_command_t;
 
-const char *svf_command_name[14] =
+static const char *svf_command_name[14] =
 {
 	"ENDDR",
 	"ENDIR",
@@ -82,7 +81,7 @@ typedef enum
 	TRST_ABSENT
 }trst_mode_t;
 
-const char *svf_trst_mode_name[4] =
+static const char *svf_trst_mode_name[4] =
 {
 	"ON",
 	"OFF",
@@ -98,7 +97,22 @@ typedef struct
 	tap_state_t paths[8];
 }svf_statemove_t;
 
-svf_statemove_t svf_statemoves[] =
+/*
+ * These paths are from the SVF specification for the STATE command, to be
+ * used when the STATE command only includes the final state.  The first
+ * element of the path is the "from" (current) state, and the last one is
+ * the "to" (target) state.
+ *
+ * All specified paths are the shortest ones in the JTAG spec, and are thus
+ * not (!!) exact matches for the paths used elsewhere in OpenOCD.  Note
+ * that PAUSE-to-PAUSE transitions all go through UPDATE and then CAPTURE,
+ * which has specific effects on the various registers; they are not NOPs.
+ *
+ * Paths to RESET are disabled here.  As elsewhere in OpenOCD, and in XSVF
+ * and many SVF implementations, we don't want to risk missing that state.
+ * To get to RESET, always we ignore the current state.
+ */
+static const svf_statemove_t svf_statemoves[] =
 {
 	// from			to				num_of_moves,	paths[8]
 //	{TAP_RESET,		TAP_RESET,		1,				{TAP_RESET}},
@@ -122,7 +136,6 @@ svf_statemove_t svf_statemoves[] =
 	{TAP_IRPAUSE,	TAP_IRPAUSE,	8,				{TAP_IRPAUSE, TAP_IREXIT2, TAP_IRUPDATE, TAP_DRSELECT, TAP_IRSELECT, TAP_IRCAPTURE, TAP_IREXIT1, TAP_IRPAUSE}}
 };
 
-char *svf_tap_state_name[TAP_NUM_STATES];
 
 #define XXR_TDI						(1 << 0)
 #define XXR_TDO						(1 << 1)
@@ -155,8 +168,8 @@ typedef struct
 	svf_xxr_para_t sdr_para;
 }svf_para_t;
 
-svf_para_t svf_para;
-const svf_para_t svf_para_init =
+static svf_para_t svf_para;
+static const svf_para_t svf_para_init =
 {
 //	frequency,	ir_end_state,	dr_end_state,	runtest_run_state,	runtest_end_state,	trst_mode
 	0,			TAP_IDLE,		TAP_IDLE,		TAP_IDLE,			TAP_IDLE,			TRST_Z,
@@ -193,8 +206,6 @@ typedef struct
 static svf_check_tdo_para_t *svf_check_tdo_para = NULL;
 static int svf_check_tdo_para_index = 0;
 
-#define dimof(a)					(sizeof(a) / sizeof((a)[0]))
-
 static int svf_read_command_from_file(int fd);
 static int svf_check_tdo(void);
 static int svf_add_check_para(uint8_t enabled, int buffer_offset, int bit_len);
@@ -222,7 +233,7 @@ int svf_register_commands(struct command_context_s *cmd_ctx)
 	return ERROR_OK;
 }
 
-void svf_free_xxd_para(svf_xxr_para_t *para)
+static void svf_free_xxd_para(svf_xxr_para_t *para)
 {
 	if (NULL != para)
 	{
@@ -249,7 +260,7 @@ void svf_free_xxd_para(svf_xxr_para_t *para)
 	}
 }
 
-unsigned svf_get_mask_u32(int bitlen)
+static unsigned svf_get_mask_u32(int bitlen)
 {
 	uint32_t bitmask;
 
@@ -269,63 +280,33 @@ unsigned svf_get_mask_u32(int bitlen)
 	return bitmask;
 }
 
-static const char* tap_state_svf_name(tap_state_t state)
-{
-	const char* ret;
-
-	switch (state)
-	{
-	case TAP_RESET:		ret = "RESET";		break;
-	case TAP_IDLE:		ret = "IDLE";		break;
-	case TAP_DRSELECT:	ret = "DRSELECT";	break;
-	case TAP_DRCAPTURE: ret = "DRCAPTURE";	break;
-	case TAP_DRSHIFT:	ret = "DRSHIFT";	break;
-	case TAP_DREXIT1:	ret = "DREXIT1";	break;
-	case TAP_DRPAUSE:	ret = "DRPAUSE";	break;
-	case TAP_DREXIT2:	ret = "DREXIT2";	break;
-	case TAP_DRUPDATE:	ret = "DRUPDATE";	break;
-	case TAP_IRSELECT:	ret = "IRSELECT";	break;
-	case TAP_IRCAPTURE: ret = "IRCAPTURE";	break;
-	case TAP_IRSHIFT:	ret = "IRSHIFT";	break;
-	case TAP_IREXIT1:	ret = "IREXIT1";	break;
-	case TAP_IRPAUSE:	ret = "IRPAUSE";	break;
-	case TAP_IREXIT2:	ret = "IREXIT2";	break;
-	case TAP_IRUPDATE:	ret = "IRUPDATE";	break;
-	default:			ret = "???";		break;
-	}
-
-	return ret;
-}
-
-static int svf_add_statemove(tap_state_t state_to)
+int svf_add_statemove(tap_state_t state_to)
 {
 	tap_state_t state_from = cmd_queue_cur_state;
 	uint8_t index;
 
-	for (index = 0; index < dimof(svf_statemoves); index++)
+	/* when resetting, be paranoid and ignore current state */
+	if (state_to == TAP_RESET) {
+		jtag_add_tlr();
+		return ERROR_OK;
+	}
+
+	for (index = 0; index < DIM(svf_statemoves); index++)
 	{
 		if ((svf_statemoves[index].from == state_from)
 			&& (svf_statemoves[index].to == state_to))
 		{
-			if (TAP_RESET == state_from)
-			{
-				jtag_add_tlr();
-				if (svf_statemoves[index].num_of_moves > 1)
-				{
-					jtag_add_pathmove(svf_statemoves[index].num_of_moves - 1, svf_statemoves[index].paths + 1);
-				}
-			}
+			/* recorded path includes current state ... avoid extra TCKs! */
+			if (svf_statemoves[index].num_of_moves > 1)
+				jtag_add_pathmove(svf_statemoves[index].num_of_moves - 1,
+						svf_statemoves[index].paths + 1);
 			else
-			{
-				if (svf_statemoves[index].num_of_moves > 0)
-				{
-					jtag_add_pathmove(svf_statemoves[index].num_of_moves, svf_statemoves[index].paths);
-				}
-			}
+				jtag_add_pathmove(svf_statemoves[index].num_of_moves,
+						svf_statemoves[index].paths);
 			return ERROR_OK;
 		}
 	}
-	LOG_ERROR("can not move to %s", tap_state_svf_name(state_to));
+	LOG_ERROR("SVF: can not move to %s", tap_state_name(state_to));
 	return ERROR_FAIL;
 }
 
@@ -414,10 +395,6 @@ static int handle_svf_command(struct command_context_s *cmd_ctx, char *cmd, char
 	svf_buffer_size = 2 * SVF_MAX_BUFFER_SIZE_TO_COMMIT;
 
 	memcpy(&svf_para, &svf_para_init, sizeof(svf_para));
-	for (i = 0; i < (int)dimof(svf_tap_state_name); i++)
-	{
-		svf_tap_state_name[i] = (char *)tap_state_svf_name(i);
-	}
 
 	// TAP_RESET
 	jtag_add_tlr();
@@ -607,14 +584,10 @@ static int svf_parse_cmd_string(char *str, int len, char **argus, int *num_of_ar
 	return ERROR_OK;
 }
 
-static int svf_tap_state_is_stable(tap_state_t state)
+bool svf_tap_state_is_stable(tap_state_t state)
 {
-	return ((TAP_RESET == state) || (TAP_IDLE == state) || (TAP_DRPAUSE == state) || (TAP_IRPAUSE == state));
-}
-
-static int svf_tap_state_is_valid(tap_state_t state)
-{
-	return state >= 0 && state < TAP_NUM_STATES;
+	return (TAP_RESET == state) || (TAP_IDLE == state)
+			|| (TAP_DRPAUSE == state) || (TAP_IRPAUSE == state);
 }
 
 static int svf_find_string_in_array(char *str, char **strs, int num_of_element)
@@ -655,8 +628,8 @@ static int svf_adjust_array_length(uint8_t **arr, int orig_bit_len, int new_bit_
 
 static int svf_copy_hexstring_to_binary(char *str, uint8_t **bin, int orig_bit_len, int bit_len)
 {
-	int i, str_len = strlen(str), str_byte_len = (bit_len + 3) >> 2, loop_cnt;
-	uint8_t ch, need_write = 1;
+	int i, str_len = strlen(str), str_hbyte_len = (bit_len + 3) >> 2;
+	uint8_t ch = 0;
 
 	if (ERROR_OK != svf_adjust_array_length(bin, orig_bit_len, bit_len))
 	{
@@ -664,75 +637,58 @@ static int svf_copy_hexstring_to_binary(char *str, uint8_t **bin, int orig_bit_l
 		return ERROR_FAIL;
 	}
 
-	if (str_byte_len > str_len)
+	for (i = 0; i < str_hbyte_len; i++)
 	{
-		loop_cnt = str_byte_len;
-	}
-	else
-	{
-		loop_cnt = str_len;
-	}
+		ch = 0;
+		while (str_len > 0)
+		{
+			ch = str[--str_len];
 
-	for (i = 0; i < loop_cnt; i++)
-	{
-		if (i < str_len)
-		{
-			ch = str[str_len - i - 1];
-			if ((ch >= '0') && (ch <= '9'))
+			if (!isblank(ch))
 			{
-				ch = ch - '0';
+				if ((ch >= '0') && (ch <= '9'))
+				{
+					ch = ch - '0';
+					break;
+				}
+				else if ((ch >= 'A') && (ch <= 'F'))
+				{
+					ch = ch - 'A' + 10;
+					break;
+				}
+				else
+				{
+					LOG_ERROR("invalid hex string");
+					return ERROR_FAIL;
+				}
 			}
-			else if ((ch >= 'A') && (ch <= 'F'))
-			{
-				ch = ch - 'A' + 10;
-			}
-			else
-			{
-				LOG_ERROR("invalid hex string");
-				return ERROR_FAIL;
-			}
-		}
-		else
-		{
+
 			ch = 0;
 		}
 
-		// check valid
-		if (i >= str_byte_len)
+		// write bin
+		if (i % 2)
 		{
-			// all data written, other data should be all '0's and needn't to be written
-			need_write = 0;
-			if (ch != 0)
-			{
-				LOG_ERROR("value execede length");
-				return ERROR_FAIL;
-			}
+			// MSB
+			(*bin)[i / 2] |= ch << 4;
 		}
-		else if (i == (str_byte_len - 1))
+		else
 		{
-			// last data byte, written if valid
-			if ((ch & ~((1 << (bit_len - 4 * i)) - 1)) != 0)
-			{
-				LOG_ERROR("value execede length");
-				return ERROR_FAIL;
-			}
+			// LSB
+			(*bin)[i / 2] = 0;
+			(*bin)[i / 2] |= ch;
 		}
+	}
 
-		if (need_write)
-		{
-			// write bin
-			if (i % 2)
-			{
-				// MSB
-				(*bin)[i / 2] |= ch << 4;
-			}
-			else
-			{
-				// LSB
-				(*bin)[i / 2] = 0;
-				(*bin)[i / 2] |= ch;
-			}
-		}
+	// consume optional leading '0' characters
+	while (str_len > 0 && str[str_len - 1] == '0')
+		str_len--;
+
+	// check valid
+	if (str_len > 0 || (ch & ~((2 << ((bit_len - 1) % 4)) - 1)) != 0)
+	{
+		LOG_ERROR("value execeeds length");
+		return ERROR_FAIL;
 	}
 
 	return ERROR_OK;
@@ -831,7 +787,12 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 		return ERROR_FAIL;
 	}
 
-	command = svf_find_string_in_array(argus[0], (char **)svf_command_name, dimof(svf_command_name));
+	/* NOTE: we're a bit loose here, because we ignore case in
+	 * TAP state names (instead of insisting on uppercase).
+	 */
+
+	command = svf_find_string_in_array(argus[0],
+			(char **)svf_command_name, DIM(svf_command_name));
 	switch (command)
 	{
 	case ENDDR:
@@ -841,23 +802,28 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 			LOG_ERROR("invalid parameter of %s", argus[0]);
 			return ERROR_FAIL;
 		}
-		i_tmp = svf_find_string_in_array(argus[1], (char **)svf_tap_state_name, dimof(svf_tap_state_name));
+
+		i_tmp = tap_state_by_name(argus[1]);
+
 		if (svf_tap_state_is_stable(i_tmp))
 		{
 			if (command == ENDIR)
 			{
 				svf_para.ir_end_state = i_tmp;
-				LOG_DEBUG("\tir_end_state = %s", svf_tap_state_name[svf_para.ir_end_state]);
+				LOG_DEBUG("\tIR end_state = %s",
+						tap_state_name(i_tmp));
 			}
 			else
 			{
 				svf_para.dr_end_state = i_tmp;
-				LOG_DEBUG("\tdr_end_state = %s", svf_tap_state_name[svf_para.dr_end_state]);
+				LOG_DEBUG("\tDR end_state = %s",
+						tap_state_name(i_tmp));
 			}
 		}
 		else
 		{
-			LOG_ERROR("%s is not valid state", argus[1]);
+			LOG_ERROR("%s: %s is not a stable state",
+					argus[0], argus[1]);
 			return ERROR_FAIL;
 		}
 		break;
@@ -1091,6 +1057,7 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 			field.num_bits = i;
 			field.out_value = &svf_tdi_buffer[svf_buffer_index];
 			field.in_value = &svf_tdi_buffer[svf_buffer_index];
+			/* NOTE:  doesn't use SVF-specified state paths */
 			jtag_add_plain_dr_scan(1, &field, svf_para.dr_end_state);
 
 			svf_buffer_index += (i + 7) >> 3;
@@ -1186,6 +1153,7 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 			field.num_bits = i;
 			field.out_value = &svf_tdi_buffer[svf_buffer_index];
 			field.in_value = &svf_tdi_buffer[svf_buffer_index];
+			/* NOTE:  doesn't use SVF-specified state paths */
 			jtag_add_plain_ir_scan(1, &field, svf_para.ir_end_state);
 
 			svf_buffer_index += (i + 7) >> 3;
@@ -1209,25 +1177,31 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 		min_time = 0;
 		max_time = 0;
 		i = 1;
+
 		// run_state
-		i_tmp = svf_find_string_in_array(argus[i], (char **)svf_tap_state_name, dimof(svf_tap_state_name));
-		if (svf_tap_state_is_valid(i_tmp))
+		i_tmp = tap_state_by_name(argus[i]);
+		if (i_tmp != TAP_INVALID)
 		{
 			if (svf_tap_state_is_stable(i_tmp))
 			{
 				svf_para.runtest_run_state = i_tmp;
 
-				// When a run_state is specified, the new  run_state becomes the default end_state
+				/* When a run_state is specified, the new
+				 * run_state becomes the default end_state.
+				 */
 				svf_para.runtest_end_state = i_tmp;
-				LOG_DEBUG("\trun_state = %s", svf_tap_state_name[svf_para.runtest_run_state]);
+				LOG_DEBUG("\trun_state = %s",
+						tap_state_name(i_tmp));
 				i++;
 			}
 			else
 			{
-				LOG_ERROR("%s is not valid state", svf_tap_state_name[i_tmp]);
+				LOG_ERROR("%s: %s is not a stable state",
+					argus[0], tap_state_name(i_tmp));
 				return ERROR_FAIL;
 			}
 		}
+
 		// run_count run_clk
 		if (((i + 2) <= num_of_argu) && strcmp(argus[i + 1], "SEC"))
 		{
@@ -1261,15 +1235,18 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 		// ENDSTATE end_state
 		if (((i + 2) <= num_of_argu) && !strcmp(argus[i], "ENDSTATE"))
 		{
-			i_tmp = svf_find_string_in_array(argus[i + 1], (char **)svf_tap_state_name, dimof(svf_tap_state_name));
+			i_tmp = tap_state_by_name(argus[i + 1]);
+
 			if (svf_tap_state_is_stable(i_tmp))
 			{
 				svf_para.runtest_end_state = i_tmp;
-				LOG_DEBUG("\tend_state = %s", svf_tap_state_name[svf_para.runtest_end_state]);
+				LOG_DEBUG("\tend_state = %s",
+					tap_state_name(i_tmp));
 			}
 			else
 			{
-				LOG_ERROR("%s is not valid state", svf_tap_state_name[i_tmp]);
+				LOG_ERROR("%s: %s is not a stable state",
+					argus[0], tap_state_name(i_tmp));
 				return ERROR_FAIL;
 			}
 			i += 2;
@@ -1287,10 +1264,13 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 				// run_state and end_state is checked to be stable state
 				// TODO: do runtest
 #if 1
+				/* FIXME handle statemove failures */
+				int retval;
+
 				// enter into run_state if necessary
 				if (cmd_queue_cur_state != svf_para.runtest_run_state)
 				{
-					svf_add_statemove(svf_para.runtest_run_state);
+					retval = svf_add_statemove(svf_para.runtest_run_state);
 				}
 
 				// call jtag_add_clocks
@@ -1299,13 +1279,13 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 				// move to end_state if necessary
 				if (svf_para.runtest_end_state != svf_para.runtest_run_state)
 				{
-					svf_add_statemove(svf_para.runtest_end_state);
+					retval = svf_add_statemove(svf_para.runtest_end_state);
 				}
 #else
 				if (svf_para.runtest_run_state != TAP_IDLE)
 				{
-					// RUNTEST can only executed in TAP_IDLE
-					LOG_ERROR("cannot runtest in %s state", svf_tap_state_name[svf_para.runtest_run_state]);
+					LOG_ERROR("cannot runtest in %s state",
+						tap_state_name(svf_para.runtest_run_state));
 					return ERROR_FAIL;
 				}
 
@@ -1336,17 +1316,21 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 				return ERROR_FAIL;
 			}
 			num_of_argu--;		// num of path
-			i_tmp = 1;			// path is from patameter 1
-			for (i = 0; i < num_of_argu; i++)
+			i_tmp = 1;		/* path is from parameter 1 */
+			for (i = 0; i < num_of_argu; i++, i_tmp++)
 			{
-				path[i] = svf_find_string_in_array(argus[i_tmp++], (char **)svf_tap_state_name, dimof(svf_tap_state_name));
-				if (!svf_tap_state_is_valid(path[i]))
+				path[i] = tap_state_by_name(argus[i_tmp]);
+				if (path[i] == TAP_INVALID)
 				{
-					LOG_ERROR("%s is not valid state", svf_tap_state_name[path[i]]);
+					LOG_ERROR("%s: %s is not a valid state",
+						argus[0], argus[i_tmp]);
+					free(path);
 					return ERROR_FAIL;
 				}
+				/* OpenOCD refuses paths containing TAP_RESET */
 				if (TAP_RESET == path[i])
 				{
+					/* FIXME last state MUST be stable! */
 					if (i > 0)
 					{
 						jtag_add_pathmove(i, path);
@@ -1362,13 +1346,16 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 				if (svf_tap_state_is_stable(path[num_of_argu - 1]))
 				{
 					// last state MUST be stable state
-					// TODO: call path_move
 					jtag_add_pathmove(num_of_argu, path);
-					LOG_DEBUG("\tmove to %s by path_move", svf_tap_state_name[path[num_of_argu - 1]]);
+					LOG_DEBUG("\tmove to %s by path_move",
+						tap_state_name(path[num_of_argu - 1]));
 				}
 				else
 				{
-					LOG_ERROR("%s is not valid state", svf_tap_state_name[path[num_of_argu - 1]]);
+					LOG_ERROR("%s: %s is not a stable state",
+						argus[0],
+						tap_state_name(path[num_of_argu - 1]));
+					free(path);
 					return ERROR_FAIL;
 				}
 			}
@@ -1382,17 +1369,18 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 		else
 		{
 			// STATE stable_state
-			state = svf_find_string_in_array(argus[1], (char **)svf_tap_state_name, dimof(svf_tap_state_name));
+			state = tap_state_by_name(argus[1]);
 			if (svf_tap_state_is_stable(state))
 			{
-				// TODO: move to state
+				LOG_DEBUG("\tmove to %s by svf_add_statemove",
+						tap_state_name(state));
+				/* FIXME handle statemove failures */
 				svf_add_statemove(state);
-
-				LOG_DEBUG("\tmove to %s by svf_add_statemove", svf_tap_state_name[state]);
 			}
 			else
 			{
-				LOG_ERROR("%s is not valid state", svf_tap_state_name[state]);
+				LOG_ERROR("%s: %s is not a stable state",
+					argus[0], tap_state_name(state));
 				return ERROR_FAIL;
 			}
 		}
@@ -1410,7 +1398,9 @@ static int svf_run_command(struct command_context_s *cmd_ctx, char *cmd_str)
 			{
 				return ERROR_FAIL;
 			}
-			i_tmp = svf_find_string_in_array(argus[1], (char **)svf_trst_mode_name, dimof(svf_trst_mode_name));
+			i_tmp = svf_find_string_in_array(argus[1],
+					(char **)svf_trst_mode_name,
+					DIM(svf_trst_mode_name));
 			switch (i_tmp)
 			{
 			case TRST_ON:
