@@ -26,11 +26,8 @@
 #include "mips32.h"
 #include "mips_m4k.h"
 #include "mips32_dmaacc.h"
-#include "jtag.h"
-#include "log.h"
+#include "target_type.h"
 
-#include <stdlib.h>
-#include <string.h>
 
 /* cli handling */
 
@@ -38,10 +35,10 @@
 int mips_m4k_poll(target_t *target);
 int mips_m4k_halt(struct target_s *target);
 int mips_m4k_soft_reset_halt(struct target_s *target);
-int mips_m4k_resume(struct target_s *target, int current, u32 address, int handle_breakpoints, int debug_execution);
-int mips_m4k_step(struct target_s *target, int current, u32 address, int handle_breakpoints);
-int mips_m4k_read_memory(struct target_s *target, u32 address, u32 size, u32 count, u8 *buffer);
-int mips_m4k_write_memory(struct target_s *target, u32 address, u32 size, u32 count, u8 *buffer);
+int mips_m4k_resume(struct target_s *target, int current, uint32_t address, int handle_breakpoints, int debug_execution);
+int mips_m4k_step(struct target_s *target, int current, uint32_t address, int handle_breakpoints);
+int mips_m4k_read_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
+int mips_m4k_write_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer);
 int mips_m4k_register_commands(struct command_context_s *cmd_ctx);
 int mips_m4k_init_target(struct command_context_s *cmd_ctx, struct target_s *target);
 int mips_m4k_quit(void);
@@ -50,6 +47,7 @@ int mips_m4k_target_create(struct target_s *target, Jim_Interp *interp);
 int mips_m4k_examine(struct target_s *target);
 int mips_m4k_assert_reset(target_t *target);
 int mips_m4k_deassert_reset(target_t *target);
+int mips_m4k_checksum_memory(target_t *target, uint32_t address, uint32_t size, uint32_t *checksum);
 
 target_type_t mips_m4k_target =
 {
@@ -67,15 +65,15 @@ target_type_t mips_m4k_target =
 	.assert_reset = mips_m4k_assert_reset,
 	.deassert_reset = mips_m4k_deassert_reset,
 	.soft_reset_halt = mips_m4k_soft_reset_halt,
-	
+
 	.get_gdb_reg_list = mips32_get_gdb_reg_list,
 
 	.read_memory = mips_m4k_read_memory,
 	.write_memory = mips_m4k_write_memory,
 	.bulk_write_memory = mips_m4k_bulk_write_memory,
-	.checksum_memory = NULL,
+	.checksum_memory = mips_m4k_checksum_memory,
 	.blank_check_memory = NULL,
-	
+
 	.run_algorithm = mips32_run_algorithm,
 
 	.add_breakpoint = mips_m4k_add_breakpoint,
@@ -90,40 +88,68 @@ target_type_t mips_m4k_target =
 	.quit = mips_m4k_quit
 };
 
-int mips_m4k_debug_entry(target_t *target)
+int mips_m4k_examine_debug_reason(target_t *target)
 {
-	u32 debug_reg;
-	mips32_common_t *mips32 = target->arch_info;
-	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	
-	/* read debug register */
-	mips_ejtag_read_debug(ejtag_info, &debug_reg);
-	
+	uint32_t break_status;
+	int retval;
+
 	if ((target->debug_reason != DBG_REASON_DBGRQ)
 		&& (target->debug_reason != DBG_REASON_SINGLESTEP))
 	{
-//		if (cortex_m3->nvic_dfsr & DFSR_BKPT)
-//		{
-//			target->debug_reason = DBG_REASON_BREAKPOINT;
-//			if (cortex_m3->nvic_dfsr & DFSR_DWTTRAP)
-//				target->debug_reason = DBG_REASON_WPTANDBKPT;
-//		}
-//		else if (cortex_m3->nvic_dfsr & DFSR_DWTTRAP)
-//			target->debug_reason = DBG_REASON_WATCHPOINT;
+		/* get info about inst breakpoint support */
+		if ((retval = target_read_u32(target, EJTAG_IBS, &break_status)) != ERROR_OK)
+			return retval;
+		if (break_status & 0x1f)
+		{
+			/* we have halted on a  breakpoint */
+			if ((retval = target_write_u32(target, EJTAG_IBS, 0)) != ERROR_OK)
+				return retval;
+			target->debug_reason = DBG_REASON_BREAKPOINT;
+		}
+
+		/* get info about data breakpoint support */
+		if ((retval = target_read_u32(target, 0xFF302000, &break_status)) != ERROR_OK)
+			return retval;
+		if (break_status & 0x1f)
+		{
+			/* we have halted on a  breakpoint */
+			if ((retval = target_write_u32(target, 0xFF302000, 0)) != ERROR_OK)
+				return retval;
+			target->debug_reason = DBG_REASON_WATCHPOINT;
+		}
 	}
-	
+
+	return ERROR_OK;
+}
+
+int mips_m4k_debug_entry(target_t *target)
+{
+	mips32_common_t *mips32 = target->arch_info;
+	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
+	uint32_t debug_reg;
+
+	/* read debug register */
+	mips_ejtag_read_debug(ejtag_info, &debug_reg);
+
+	/* make sure break uit configured */
+	mips32_configure_break_unit(target);
+
+	/* attempt to find halt reason */
+	mips_m4k_examine_debug_reason(target);
+
+	/* clear single step if active */
 	if (debug_reg & EJTAG_DEBUG_DSS)
 	{
 		/* stopped due to single step - clear step bit */
 		mips_ejtag_config_step(ejtag_info, 0);
 	}
-	
+
 	mips32_save_context(target);
-	
-	LOG_DEBUG("entered debug state at PC 0x%x, target->state: %s", 
-		*(u32*)(mips32->core_cache->reg_list[MIPS32_PC].value), 
-		  Jim_Nvp_value2name_simple( nvp_target_state, target->state )->name);
-	
+
+	LOG_DEBUG("entered debug state at PC 0x%" PRIx32 ", target->state: %s",
+		*(uint32_t*)(mips32->core_cache->reg_list[MIPS32_PC].value),
+		  target_state_name(target));
+
 	return ERROR_OK;
 }
 
@@ -132,34 +158,49 @@ int mips_m4k_poll(target_t *target)
 	int retval;
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	u32 ejtag_ctrl = ejtag_info->ejtag_ctrl;
-	
+	uint32_t ejtag_ctrl = ejtag_info->ejtag_ctrl;
+
 	/* read ejtag control reg */
-	jtag_add_end_state(TAP_RTI);
+	jtag_set_end_state(TAP_IDLE);
 	mips_ejtag_set_instr(ejtag_info, EJTAG_INST_CONTROL, NULL);
 	mips_ejtag_drscan_32(ejtag_info, &ejtag_ctrl);
-	
+
+	/* clear this bit before handling polling
+	 * as after reset registers will read zero */
+	if (ejtag_ctrl & EJTAG_CTRL_ROCC)
+	{
+		/* we have detected a reset, clear flag
+		 * otherwise ejtag will not work */
+		jtag_set_end_state(TAP_IDLE);
+		ejtag_ctrl = ejtag_info->ejtag_ctrl & ~EJTAG_CTRL_ROCC;
+
+		mips_ejtag_set_instr(ejtag_info, EJTAG_INST_CONTROL, NULL);
+		mips_ejtag_drscan_32(ejtag_info, &ejtag_ctrl);
+		LOG_DEBUG("Reset Detected");
+	}
+
+	/* check for processor halted */
 	if (ejtag_ctrl & EJTAG_CTRL_BRKST)
 	{
 		if ((target->state == TARGET_RUNNING) || (target->state == TARGET_RESET))
 		{
-			jtag_add_end_state(TAP_RTI);
+			jtag_set_end_state(TAP_IDLE);
 			mips_ejtag_set_instr(ejtag_info, EJTAG_INST_NORMALBOOT, NULL);
-			
+
 			target->state = TARGET_HALTED;
-			
+
 			if ((retval = mips_m4k_debug_entry(target)) != ERROR_OK)
 				return retval;
-			
+
 			target_call_event_callbacks(target, TARGET_EVENT_HALTED);
 		}
 		else if (target->state == TARGET_DEBUG_RUNNING)
 		{
 			target->state = TARGET_HALTED;
-			
+
 			if ((retval = mips_m4k_debug_entry(target)) != ERROR_OK)
 				return retval;
-			
+
 			target_call_event_callbacks(target, TARGET_EVENT_DEBUG_HALTED);
 		}
 	}
@@ -167,21 +208,9 @@ int mips_m4k_poll(target_t *target)
 	{
 		target->state = TARGET_RUNNING;
 	}
-	
-	if (ejtag_ctrl & EJTAG_CTRL_ROCC)
-	{
-		/* we have detected a reset, clear flag
-		 * otherwise ejtag will not work */
-		jtag_add_end_state(TAP_RTI);
-		ejtag_ctrl = ejtag_info->ejtag_ctrl & ~EJTAG_CTRL_ROCC;
-		
-		mips_ejtag_set_instr(ejtag_info, EJTAG_INST_CONTROL, NULL);
-		mips_ejtag_drscan_32(ejtag_info, &ejtag_ctrl);
-		LOG_DEBUG("Reset Detected");
-	}
-	
-//	LOG_DEBUG("ctrl=0x%08X", ejtag_ctrl);
-	
+
+//	LOG_DEBUG("ctrl = 0x%08X", ejtag_ctrl);
+
 	return ERROR_OK;
 }
 
@@ -189,24 +218,24 @@ int mips_m4k_halt(struct target_s *target)
 {
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	
-	LOG_DEBUG("target->state: %s", 
-		  Jim_Nvp_value2name_simple( nvp_target_state, target->state )->name);
-	
+
+	LOG_DEBUG("target->state: %s",
+		  target_state_name(target));
+
 	if (target->state == TARGET_HALTED)
 	{
 		LOG_DEBUG("target was already halted");
 		return ERROR_OK;
 	}
-	
+
 	if (target->state == TARGET_UNKNOWN)
 	{
 		LOG_WARNING("target was in unknown state when halt was requested");
 	}
-	
-	if (target->state == TARGET_RESET) 
+
+	if (target->state == TARGET_RESET)
 	{
-		if ((jtag_reset_config & RESET_SRST_PULLS_TRST) && jtag_srst)
+		if ((jtag_get_reset_config() & RESET_SRST_PULLS_TRST) && jtag_get_srst())
 		{
 			LOG_ERROR("can't request a halt while in reset if nSRST pulls nTRST");
 			return ERROR_TARGET_FAILURE;
@@ -217,16 +246,16 @@ int mips_m4k_halt(struct target_s *target)
 			 * debug entry was already prepared in mips32_prepare_reset_halt()
 			 */
 			target->debug_reason = DBG_REASON_DBGRQ;
-			
+
 			return ERROR_OK;
 		}
 	}
-	
+
 	/* break processor */
 	mips_ejtag_enter_debug(ejtag_info);
-	
+
 	target->debug_reason = DBG_REASON_DBGRQ;
-	
+
 	return ERROR_OK;
 }
 
@@ -234,34 +263,38 @@ int mips_m4k_assert_reset(target_t *target)
 {
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	
-	LOG_DEBUG("target->state: %s", 
-		Jim_Nvp_value2name_simple( nvp_target_state, target->state )->name);
-	
+
+	LOG_DEBUG("target->state: %s",
+		target_state_name(target));
+
+	enum reset_types jtag_reset_config = jtag_get_reset_config();
 	if (!(jtag_reset_config & RESET_HAS_SRST))
 	{
 		LOG_ERROR("Can't assert SRST");
 		return ERROR_FAIL;
 	}
-	
+
 	if (target->reset_halt)
 	{
 		/* use hardware to catch reset */
-		jtag_add_end_state(TAP_RTI);
+		jtag_set_end_state(TAP_IDLE);
 		mips_ejtag_set_instr(ejtag_info, EJTAG_INST_EJTAGBOOT, NULL);
 	}
 	else
 	{
-		jtag_add_end_state(TAP_RTI);
+		jtag_set_end_state(TAP_IDLE);
 		mips_ejtag_set_instr(ejtag_info, EJTAG_INST_NORMALBOOT, NULL);
 	}
-	
-	if (strcmp(target->variant, "ejtag_srst") == 0) {
-		u32 ejtag_ctrl = ejtag_info->ejtag_ctrl | EJTAG_CTRL_PRRST | EJTAG_CTRL_PERRST;
+
+	if (strcmp(target->variant, "ejtag_srst") == 0)
+	{
+		uint32_t ejtag_ctrl = ejtag_info->ejtag_ctrl | EJTAG_CTRL_PRRST | EJTAG_CTRL_PERRST;
 		LOG_DEBUG("Using EJTAG reset (PRRST) to reset processor...");
 		mips_ejtag_set_instr(ejtag_info, EJTAG_INST_CONTROL, NULL);
 		mips_ejtag_drscan_32(ejtag_info, &ejtag_ctrl);
-	} else {
+	}
+	else
+	{
 		/* here we should issue a srst only, but we may have to assert trst as well */
 		if (jtag_reset_config & RESET_SRST_PULLS_TRST)
 		{
@@ -272,7 +305,7 @@ int mips_m4k_assert_reset(target_t *target)
 			jtag_add_reset(0, 1);
 		}
 	}
-	
+
 	target->state = TARGET_RESET;
 	jtag_add_sleep(50000);
 
@@ -281,22 +314,21 @@ int mips_m4k_assert_reset(target_t *target)
 	if (target->reset_halt)
 	{
 		int retval;
-		if ((retval = target_halt(target))!=ERROR_OK)
+		if ((retval = target_halt(target)) != ERROR_OK)
 			return retval;
 	}
-	
-	
+
 	return ERROR_OK;
 }
 
 int mips_m4k_deassert_reset(target_t *target)
 {
-	LOG_DEBUG("target->state: %s", 
-		Jim_Nvp_value2name_simple( nvp_target_state, target->state )->name);
-	
+	LOG_DEBUG("target->state: %s",
+		target_state_name(target));
+
 	/* deassert reset lines */
 	jtag_add_reset(0, 0);
-	
+
 	return ERROR_OK;
 }
 
@@ -306,26 +338,45 @@ int mips_m4k_soft_reset_halt(struct target_s *target)
 	return ERROR_OK;
 }
 
-int mips_m4k_resume(struct target_s *target, int current, u32 address, int handle_breakpoints, int debug_execution)
+int mips_m4k_single_step_core(target_t *target)
+{
+	mips32_common_t *mips32 = target->arch_info;
+	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
+
+	/* configure single step mode */
+	mips_ejtag_config_step(ejtag_info, 1);
+
+	/* disable interrupts while stepping */
+	mips32_enable_interrupts(target, 0);
+
+	/* exit debug mode */
+	mips_ejtag_exit_debug(ejtag_info);
+
+	mips_m4k_debug_entry(target);
+
+	return ERROR_OK;
+}
+
+int mips_m4k_resume(struct target_s *target, int current, uint32_t address, int handle_breakpoints, int debug_execution)
 {
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
 	breakpoint_t *breakpoint = NULL;
-	u32 resume_pc;
-	
+	uint32_t resume_pc;
+
 	if (target->state != TARGET_HALTED)
 	{
 		LOG_WARNING("target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
-	
+
 	if (!debug_execution)
 	{
 		target_free_all_working_areas(target);
 		mips_m4k_enable_breakpoints(target);
 		mips_m4k_enable_watchpoints(target);
 	}
-	
+
 	/* current = 1: continue on current pc, otherwise continue at <address> */
 	if (!current)
 	{
@@ -333,47 +384,51 @@ int mips_m4k_resume(struct target_s *target, int current, u32 address, int handl
 		mips32->core_cache->reg_list[MIPS32_PC].dirty = 1;
 		mips32->core_cache->reg_list[MIPS32_PC].valid = 1;
 	}
-	
+
 	resume_pc = buf_get_u32(mips32->core_cache->reg_list[MIPS32_PC].value, 0, 32);
-	
+
 	mips32_restore_context(target);
-	
+
 	/* the front-end may request us not to handle breakpoints */
 	if (handle_breakpoints)
 	{
 		/* Single step past breakpoint at current address */
 		if ((breakpoint = breakpoint_find(target, resume_pc)))
 		{
-			LOG_DEBUG("unset breakpoint at 0x%8.8x", breakpoint->address);
+			LOG_DEBUG("unset breakpoint at 0x%8.8" PRIx32 "", breakpoint->address);
 			mips_m4k_unset_breakpoint(target, breakpoint);
-			//mips_m4k_single_step_core(target);
+			mips_m4k_single_step_core(target);
 			mips_m4k_set_breakpoint(target, breakpoint);
 		}
 	}
-	
-	/* exit debug mode - enable interrupts if required */
-	mips_ejtag_exit_debug(ejtag_info, !debug_execution);
-	
+
+	/* enable interrupts if we are running */
+	mips32_enable_interrupts(target, !debug_execution);
+
+	/* exit debug mode */
+	mips_ejtag_exit_debug(ejtag_info);
+	target->debug_reason = DBG_REASON_NOTHALTED;
+
 	/* registers are now invalid */
 	mips32_invalidate_core_regs(target);
-	
+
 	if (!debug_execution)
 	{
 		target->state = TARGET_RUNNING;
 		target_call_event_callbacks(target, TARGET_EVENT_RESUMED);
-		LOG_DEBUG("target resumed at 0x%x", resume_pc);
+		LOG_DEBUG("target resumed at 0x%" PRIx32 "", resume_pc);
 	}
 	else
 	{
 		target->state = TARGET_DEBUG_RUNNING;
 		target_call_event_callbacks(target, TARGET_EVENT_DEBUG_RESUMED);
-		LOG_DEBUG("target debug resumed at 0x%x", resume_pc);
+		LOG_DEBUG("target debug resumed at 0x%" PRIx32 "", resume_pc);
 	}
-	
+
 	return ERROR_OK;
 }
 
-int mips_m4k_step(struct target_s *target, int current, u32 address, int handle_breakpoints)
+int mips_m4k_step(struct target_s *target, int current, uint32_t address, int handle_breakpoints)
 {
 	/* get pointers to arch-specific information */
 	mips32_common_t *mips32 = target->arch_info;
@@ -389,43 +444,46 @@ int mips_m4k_step(struct target_s *target, int current, u32 address, int handle_
 	/* current = 1: continue on current pc, otherwise continue at <address> */
 	if (!current)
 		buf_set_u32(mips32->core_cache->reg_list[MIPS32_PC].value, 0, 32, address);
-	
+
 	/* the front-end may request us not to handle breakpoints */
 	if (handle_breakpoints)
 		if ((breakpoint = breakpoint_find(target, buf_get_u32(mips32->core_cache->reg_list[MIPS32_PC].value, 0, 32))))
 			mips_m4k_unset_breakpoint(target, breakpoint);
-	
+
 	/* restore context */
 	mips32_restore_context(target);
-	
+
 	/* configure single step mode */
 	mips_ejtag_config_step(ejtag_info, 1);
-	
+
 	target->debug_reason = DBG_REASON_SINGLESTEP;
-	
+
 	target_call_event_callbacks(target, TARGET_EVENT_RESUMED);
-	
+
+	/* disable interrupts while stepping */
+	mips32_enable_interrupts(target, 0);
+
 	/* exit debug mode */
-	mips_ejtag_exit_debug(ejtag_info, 1);
-	
+	mips_ejtag_exit_debug(ejtag_info);
+
 	/* registers are now invalid */
 	mips32_invalidate_core_regs(target);
-	
+
 	if (breakpoint)
 		mips_m4k_set_breakpoint(target, breakpoint);
 
 	LOG_DEBUG("target stepped ");
-	
+
 	mips_m4k_debug_entry(target);
 	target_call_event_callbacks(target, TARGET_EVENT_HALTED);
-	
+
 	return ERROR_OK;
 }
 
 void mips_m4k_enable_breakpoints(struct target_s *target)
 {
 	breakpoint_t *breakpoint = target->breakpoints;
-	
+
 	/* set any pending breakpoints */
 	while (breakpoint)
 	{
@@ -437,56 +495,347 @@ void mips_m4k_enable_breakpoints(struct target_s *target)
 
 int mips_m4k_set_breakpoint(struct target_s *target, breakpoint_t *breakpoint)
 {
-	/* TODO */
+	mips32_common_t *mips32 = target->arch_info;
+	mips32_comparator_t * comparator_list = mips32->inst_break_list;
+	int retval;
+
+	if (breakpoint->set)
+	{
+		LOG_WARNING("breakpoint already set");
+		return ERROR_OK;
+	}
+
+	if (breakpoint->type == BKPT_HARD)
+	{
+		int bp_num = 0;
+
+		while (comparator_list[bp_num].used && (bp_num < mips32->num_inst_bpoints))
+			bp_num++;
+		if (bp_num >= mips32->num_inst_bpoints)
+		{
+			LOG_DEBUG("ERROR Can not find free FP Comparator(bpid: %d)",
+					  breakpoint->unique_id );
+			LOG_WARNING("ERROR Can not find free FP Comparator");
+			exit(-1);
+		}
+		breakpoint->set = bp_num + 1;
+		comparator_list[bp_num].used = 1;
+		comparator_list[bp_num].bp_value = breakpoint->address;
+		target_write_u32(target, comparator_list[bp_num].reg_address, comparator_list[bp_num].bp_value);
+		target_write_u32(target, comparator_list[bp_num].reg_address + 0x08, 0x00000000);
+		target_write_u32(target, comparator_list[bp_num].reg_address + 0x18, 1);
+		LOG_DEBUG("bpid: %d, bp_num %i bp_value 0x%" PRIx32 "",
+				  breakpoint->unique_id,
+				  bp_num, comparator_list[bp_num].bp_value);
+	}
+	else if (breakpoint->type == BKPT_SOFT)
+	{
+		LOG_DEBUG("bpid: %d", breakpoint->unique_id );
+		if (breakpoint->length == 4)
+		{
+			uint32_t verify = 0xffffffff;
+
+			if ((retval = target_read_memory(target, breakpoint->address, breakpoint->length, 1, breakpoint->orig_instr)) != ERROR_OK)
+			{
+				return retval;
+			}
+			if ((retval = target_write_u32(target, breakpoint->address, MIPS32_SDBBP)) != ERROR_OK)
+			{
+				return retval;
+			}
+
+			if ((retval = target_read_u32(target, breakpoint->address, &verify)) != ERROR_OK)
+			{
+				return retval;
+			}
+			if (verify != MIPS32_SDBBP)
+			{
+				LOG_ERROR("Unable to set 32bit breakpoint at address %08" PRIx32 " - check that memory is read/writable", breakpoint->address);
+				return ERROR_OK;
+			}
+		}
+		else
+		{
+			uint16_t verify = 0xffff;
+
+			if ((retval = target_read_memory(target, breakpoint->address, breakpoint->length, 1, breakpoint->orig_instr)) != ERROR_OK)
+			{
+				return retval;
+			}
+			if ((retval = target_write_u16(target, breakpoint->address, MIPS16_SDBBP)) != ERROR_OK)
+			{
+				return retval;
+			}
+
+			if ((retval = target_read_u16(target, breakpoint->address, &verify)) != ERROR_OK)
+			{
+				return retval;
+			}
+			if (verify != MIPS16_SDBBP)
+			{
+				LOG_ERROR("Unable to set 16bit breakpoint at address %08" PRIx32 " - check that memory is read/writable", breakpoint->address);
+				return ERROR_OK;
+			}
+		}
+
+		breakpoint->set = 20; /* Any nice value but 0 */
+	}
+
 	return ERROR_OK;
 }
 
 int mips_m4k_unset_breakpoint(struct target_s *target, breakpoint_t *breakpoint)
 {
-	/* TODO */
+	/* get pointers to arch-specific information */
+	mips32_common_t *mips32 = target->arch_info;
+	mips32_comparator_t * comparator_list = mips32->inst_break_list;
+	int retval;
+
+	if (!breakpoint->set)
+	{
+		LOG_WARNING("breakpoint not set");
+		return ERROR_OK;
+	}
+
+	if (breakpoint->type == BKPT_HARD)
+	{
+		int bp_num = breakpoint->set - 1;
+		if ((bp_num < 0) || (bp_num >= mips32->num_inst_bpoints))
+		{
+			LOG_DEBUG("Invalid FP Comparator number in breakpoint (bpid: %d)",
+					  breakpoint->unique_id);
+			return ERROR_OK;
+		}
+		LOG_DEBUG("bpid: %d - releasing hw: %d",
+				  breakpoint->unique_id,
+				  bp_num );
+		comparator_list[bp_num].used = 0;
+		comparator_list[bp_num].bp_value = 0;
+		target_write_u32(target, comparator_list[bp_num].reg_address + 0x18, 0);
+
+	}
+	else
+	{
+		/* restore original instruction (kept in target endianness) */
+		LOG_DEBUG("bpid: %d", breakpoint->unique_id);
+		if (breakpoint->length == 4)
+		{
+			uint32_t current_instr;
+
+			/* check that user program has not modified breakpoint instruction */
+			if ((retval = target_read_memory(target, breakpoint->address, 4, 1, (uint8_t*)&current_instr)) != ERROR_OK)
+			{
+				return retval;
+			}
+			if (current_instr == MIPS32_SDBBP)
+			{
+				if ((retval = target_write_memory(target, breakpoint->address, 4, 1, breakpoint->orig_instr)) != ERROR_OK)
+				{
+					return retval;
+				}
+			}
+		}
+		else
+		{
+			uint16_t current_instr;
+
+			/* check that user program has not modified breakpoint instruction */
+			if ((retval = target_read_memory(target, breakpoint->address, 2, 1, (uint8_t*)&current_instr)) != ERROR_OK)
+			{
+				return retval;
+			}
+
+			if (current_instr == MIPS16_SDBBP)
+			{
+				if ((retval = target_write_memory(target, breakpoint->address, 2, 1, breakpoint->orig_instr)) != ERROR_OK)
+				{
+					return retval;
+				}
+			}
+		}
+	}
+	breakpoint->set = 0;
+
 	return ERROR_OK;
 }
 
 int mips_m4k_add_breakpoint(struct target_s *target, breakpoint_t *breakpoint)
 {
-	/* TODO */
+	mips32_common_t *mips32 = target->arch_info;
+
+	if (breakpoint->type == BKPT_HARD)
+	{
+		if (mips32->num_inst_bpoints_avail < 1)
+		{
+			LOG_INFO("no hardware breakpoint available");
+			return ERROR_TARGET_RESOURCE_NOT_AVAILABLE;
+		}
+
+		mips32->num_inst_bpoints_avail--;
+	}
+
+	mips_m4k_set_breakpoint(target, breakpoint);
+
 	return ERROR_OK;
 }
 
 int mips_m4k_remove_breakpoint(struct target_s *target, breakpoint_t *breakpoint)
 {
-	/* TODO */
+	/* get pointers to arch-specific information */
+	mips32_common_t *mips32 = target->arch_info;
+
+	if (target->state != TARGET_HALTED)
+	{
+		LOG_WARNING("target not halted");
+		return ERROR_TARGET_NOT_HALTED;
+	}
+
+	if (breakpoint->set)
+	{
+		mips_m4k_unset_breakpoint(target, breakpoint);
+	}
+
+	if (breakpoint->type == BKPT_HARD)
+		mips32->num_inst_bpoints_avail++;
+
 	return ERROR_OK;
 }
 
 int mips_m4k_set_watchpoint(struct target_s *target, watchpoint_t *watchpoint)
 {
-	/* TODO */
+	mips32_common_t *mips32 = target->arch_info;
+	mips32_comparator_t * comparator_list = mips32->data_break_list;
+	int wp_num = 0;
+	/*
+	 * watchpoint enabled, ignore all byte lanes in value register
+	 * and exclude both load and store accesses from  watchpoint
+	 * condition evaluation
+	*/
+	int enable = EJTAG_DBCn_NOSB | EJTAG_DBCn_NOLB | EJTAG_DBCn_BE |
+                (0xff << EJTAG_DBCn_BLM_SHIFT);
+
+	if (watchpoint->set)
+	{
+		LOG_WARNING("watchpoint already set");
+		return ERROR_OK;
+	}
+
+	while(comparator_list[wp_num].used && (wp_num < mips32->num_data_bpoints))
+		wp_num++;
+	if (wp_num >= mips32->num_data_bpoints)
+	{
+		LOG_DEBUG("ERROR Can not find free FP Comparator");
+		LOG_WARNING("ERROR Can not find free FP Comparator");
+		exit(-1);
+	}
+
+	if (watchpoint->length != 4)
+	{
+		LOG_ERROR("Only watchpoints of length 4 are supported");
+		return ERROR_TARGET_UNALIGNED_ACCESS;
+	}
+
+	if (watchpoint->address % 4)
+	{
+		LOG_ERROR("Watchpoints address should be word aligned");
+		return ERROR_TARGET_UNALIGNED_ACCESS;
+	}
+
+	switch (watchpoint->rw)
+	{
+		case WPT_READ:
+			enable &= ~EJTAG_DBCn_NOLB;
+			break;
+		case WPT_WRITE:
+			enable &= ~EJTAG_DBCn_NOSB;
+			break;
+		case WPT_ACCESS:
+			enable &= ~(EJTAG_DBCn_NOLB | EJTAG_DBCn_NOSB);
+			break;
+		default:
+			LOG_ERROR("BUG: watchpoint->rw neither read, write nor access");
+	}
+
+	watchpoint->set = wp_num + 1;
+	comparator_list[wp_num].used = 1;
+	comparator_list[wp_num].bp_value = watchpoint->address;
+	target_write_u32(target, comparator_list[wp_num].reg_address, comparator_list[wp_num].bp_value);
+	target_write_u32(target, comparator_list[wp_num].reg_address + 0x08, 0x00000000);
+	target_write_u32(target, comparator_list[wp_num].reg_address + 0x10, 0x00000000);
+	target_write_u32(target, comparator_list[wp_num].reg_address + 0x18, enable);
+	target_write_u32(target, comparator_list[wp_num].reg_address + 0x20, 0);
+	LOG_DEBUG("wp_num %i bp_value 0x%" PRIx32 "", wp_num, comparator_list[wp_num].bp_value);
+
 	return ERROR_OK;
 }
 
 int mips_m4k_unset_watchpoint(struct target_s *target, watchpoint_t *watchpoint)
 {
-	/* TODO */
+	/* get pointers to arch-specific information */
+	mips32_common_t *mips32 = target->arch_info;
+	mips32_comparator_t * comparator_list = mips32->data_break_list;
+
+	if (!watchpoint->set)
+	{
+		LOG_WARNING("watchpoint not set");
+		return ERROR_OK;
+	}
+
+	int wp_num = watchpoint->set - 1;
+	if ((wp_num < 0) || (wp_num >= mips32->num_data_bpoints))
+	{
+		LOG_DEBUG("Invalid FP Comparator number in watchpoint");
+		return ERROR_OK;
+	}
+	comparator_list[wp_num].used = 0;
+	comparator_list[wp_num].bp_value = 0;
+	target_write_u32(target, comparator_list[wp_num].reg_address + 0x18, 0);
+	watchpoint->set = 0;
+
 	return ERROR_OK;
 }
 
 int mips_m4k_add_watchpoint(struct target_s *target, watchpoint_t *watchpoint)
 {
-	/* TODO */
+	mips32_common_t *mips32 = target->arch_info;
+
+	if (mips32->num_data_bpoints_avail < 1)
+	{
+		LOG_INFO("no hardware watchpoints available");
+		return ERROR_TARGET_RESOURCE_NOT_AVAILABLE;
+	}
+
+	mips32->num_data_bpoints_avail--;
+
+	mips_m4k_set_watchpoint(target, watchpoint);
 	return ERROR_OK;
 }
 
 int mips_m4k_remove_watchpoint(struct target_s *target, watchpoint_t *watchpoint)
 {
-	/* TODO */
+	/* get pointers to arch-specific information */
+	mips32_common_t *mips32 = target->arch_info;
+
+	if (target->state != TARGET_HALTED)
+	{
+		LOG_WARNING("target not halted");
+		return ERROR_TARGET_NOT_HALTED;
+	}
+
+	if (watchpoint->set)
+	{
+		mips_m4k_unset_watchpoint(target, watchpoint);
+	}
+
+	mips32->num_data_bpoints_avail++;
+
 	return ERROR_OK;
 }
 
 void mips_m4k_enable_watchpoints(struct target_s *target)
 {
 	watchpoint_t *watchpoint = target->watchpoints;
-	
+
 	/* set any pending watchpoints */
 	while (watchpoint)
 	{
@@ -496,12 +845,12 @@ void mips_m4k_enable_watchpoints(struct target_s *target)
 	}
 }
 
-int mips_m4k_read_memory(struct target_s *target, u32 address, u32 size, u32 count, u8 *buffer)
+int mips_m4k_read_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer)
 {
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	
-	LOG_DEBUG("address: 0x%8.8x, size: 0x%8.8x, count: 0x%8.8x", address, size, count);
+
+	LOG_DEBUG("address: 0x%8.8" PRIx32 ", size: 0x%8.8" PRIx32 ", count: 0x%8.8" PRIx32 "", address, size, count);
 
 	if (target->state != TARGET_HALTED)
 	{
@@ -515,32 +864,25 @@ int mips_m4k_read_memory(struct target_s *target, u32 address, u32 size, u32 cou
 
 	if (((size == 4) && (address & 0x3u)) || ((size == 2) && (address & 0x1u)))
 		return ERROR_TARGET_UNALIGNED_ACCESS;
-	
-	switch (size)
-	{
-		case 4:
-		case 2:
-		case 1:
-			/* if noDMA off, use DMAACC mode for memory read */
-			if(ejtag_info->impcode & (1<<14))
-				return mips32_pracc_read_mem(ejtag_info, address, size, count, (void *)buffer);
-			else
-				return mips32_dmaacc_read_mem(ejtag_info, address, size, count, (void *)buffer);
-		default:
-			LOG_ERROR("BUG: we shouldn't get here");
-			exit(-1);
-			break;
-	}
+
+	/* if noDMA off, use DMAACC mode for memory read */
+	int retval;
+	if (ejtag_info->impcode & EJTAG_IMP_NODMA)
+		retval = mips32_pracc_read_mem(ejtag_info, address, size, count, (void *)buffer);
+	else
+		retval = mips32_dmaacc_read_mem(ejtag_info, address, size, count, (void *)buffer);
+	if (ERROR_OK != retval)
+		return retval;
 
 	return ERROR_OK;
 }
 
-int mips_m4k_write_memory(struct target_s *target, u32 address, u32 size, u32 count, u8 *buffer)
+int mips_m4k_write_memory(struct target_s *target, uint32_t address, uint32_t size, uint32_t count, uint8_t *buffer)
 {
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	
-	LOG_DEBUG("address: 0x%8.8x, size: 0x%8.8x, count: 0x%8.8x", address, size, count);
+
+	LOG_DEBUG("address: 0x%8.8" PRIx32 ", size: 0x%8.8" PRIx32 ", count: 0x%8.8" PRIx32 "", address, size, count);
 
 	if (target->state != TARGET_HALTED)
 	{
@@ -554,31 +896,18 @@ int mips_m4k_write_memory(struct target_s *target, u32 address, u32 size, u32 co
 
 	if (((size == 4) && (address & 0x3u)) || ((size == 2) && (address & 0x1u)))
 		return ERROR_TARGET_UNALIGNED_ACCESS;
-	
-	switch (size)
-	{
-		case 4:
-		case 2:
-		case 1:
-			/* if noDMA off, use DMAACC mode for memory write */
-			if(ejtag_info->impcode & (1<<14))
-				mips32_pracc_write_mem(ejtag_info, address, size, count, (void *)buffer);
-			else
-				mips32_dmaacc_write_mem(ejtag_info, address, size, count, (void *)buffer);
-			break;
-		default:
-			LOG_ERROR("BUG: we shouldn't get here");
-			exit(-1);
-			break;
-	}
-	
-	return ERROR_OK;
+
+	/* if noDMA off, use DMAACC mode for memory write */
+	if (ejtag_info->impcode & EJTAG_IMP_NODMA)
+		return mips32_pracc_write_mem(ejtag_info, address, size, count, (void *)buffer);
+	else
+		return mips32_dmaacc_write_mem(ejtag_info, address, size, count, (void *)buffer);
 }
 
 int mips_m4k_register_commands(struct command_context_s *cmd_ctx)
 {
 	int retval;
-	
+
 	retval = mips32_register_commands(cmd_ctx);
 	return retval;
 }
@@ -586,7 +915,7 @@ int mips_m4k_register_commands(struct command_context_s *cmd_ctx)
 int mips_m4k_init_target(struct command_context_s *cmd_ctx, struct target_s *target)
 {
 	mips32_build_reg_cache(target);
-	
+
 	return ERROR_OK;
 }
 
@@ -595,34 +924,25 @@ int mips_m4k_quit(void)
 	return ERROR_OK;
 }
 
-int mips_m4k_init_arch_info(target_t *target, mips_m4k_common_t *mips_m4k, int chain_pos, const char *variant)
+int mips_m4k_init_arch_info(target_t *target, mips_m4k_common_t *mips_m4k, jtag_tap_t *tap)
 {
 	mips32_common_t *mips32 = &mips_m4k->mips32_common;
-	
-	if (variant)
-	{
-		mips_m4k->variant = strdup(variant);
-	}
-	else
-	{
-		mips_m4k->variant = strdup("");
-	}
-	
+
 	mips_m4k->common_magic = MIPSM4K_COMMON_MAGIC;
-	
+
 	/* initialize mips4k specific info */
-	mips32_init_arch_info(target, mips32, chain_pos, variant);
+	mips32_init_arch_info(target, mips32, tap);
 	mips32->arch_info = mips_m4k;
-	
+
 	return ERROR_OK;
 }
 
 int mips_m4k_target_create(struct target_s *target, Jim_Interp *interp)
 {
 	mips_m4k_common_t *mips_m4k = calloc(1,sizeof(mips_m4k_common_t));
-	
-	mips_m4k_init_arch_info(target, mips_m4k, target->chain_position, target->variant);
-	
+
+	mips_m4k_init_arch_info(target, mips_m4k, target->tap);
+
 	return ERROR_OK;
 }
 
@@ -631,28 +951,38 @@ int mips_m4k_examine(struct target_s *target)
 	int retval;
 	mips32_common_t *mips32 = target->arch_info;
 	mips_ejtag_t *ejtag_info = &mips32->ejtag_info;
-	u32 idcode = 0;
-	
-	target->type->examined = 1;
-	
-	mips_ejtag_get_idcode(ejtag_info, &idcode, NULL);
-	
-	if (((idcode >> 1) & 0x7FF) == 0x29)
+	uint32_t idcode = 0;
+
+	if (!target_was_examined(target))
 	{
-		/* we are using a pic32mx so select ejtag port
-		 * as it is not selected by default */
-		mips_ejtag_set_instr(ejtag_info, 0x05, NULL);
-		LOG_DEBUG("PIC32MX Detected - using EJTAG Interface");
+		mips_ejtag_get_idcode(ejtag_info, &idcode);
+		ejtag_info->idcode = idcode;
+
+		if (((idcode >> 1) & 0x7FF) == 0x29)
+		{
+			/* we are using a pic32mx so select ejtag port
+			 * as it is not selected by default */
+			mips_ejtag_set_instr(ejtag_info, 0x05, NULL);
+			LOG_DEBUG("PIC32MX Detected - using EJTAG Interface");
+		}
 	}
-	
+
 	/* init rest of ejtag interface */
 	if ((retval = mips_ejtag_init(ejtag_info)) != ERROR_OK)
 		return retval;
-	
+
+	if ((retval = mips32_examine(target)) != ERROR_OK)
+		return retval;
+
 	return ERROR_OK;
 }
 
-int mips_m4k_bulk_write_memory(target_t *target, u32 address, u32 count, u8 *buffer)
+int mips_m4k_bulk_write_memory(target_t *target, uint32_t address, uint32_t count, uint8_t *buffer)
 {
 	return mips_m4k_write_memory(target, address, 4, count, buffer);
 }
+
+int mips_m4k_checksum_memory(target_t *target, uint32_t address, uint32_t size, uint32_t *checksum)
+{
+	return ERROR_FAIL; /* use bulk read method */
+}
